@@ -24,23 +24,19 @@ namespace PowerDevil
 {
 
 BrightnessLogic::BrightnessLogic()
- : m_value(-1), m_valueMax(-1), m_step(-1), m_stepMax(-1)
 {
 }
 
-BrightnessLogic::~BrightnessLogic()
+void BrightnessLogic::setValue(int value)
 {
-}
-
-void BrightnessLogic::setValue(int value) {
     m_value = value;
-    m_step = valueToStep(value);
 }
 
-void BrightnessLogic::setValueMax(int valueMax) {
+void BrightnessLogic::setValueMax(int valueMax)
+{
     if (valueMax != m_valueMax) {
         m_valueMax = valueMax;
-        m_stepMax = calculateStepMax(valueMax);
+        m_steps = calculateSteps(valueMax);
     }
 }
 
@@ -57,67 +53,83 @@ int BrightnessLogic::action(BrightnessKeyType type) const {
     return -1; // We shouldn't get here
 }
 
-int BrightnessLogic::increased() const {
+int BrightnessLogic::increased() const
+{
     if (m_value == m_valueMax) {
         return -1; // ignore, we are at the maximum
     }
 
     // Add 1 and round upwards to the nearest step
-    int step = m_stepMax - (m_valueMax - m_value - 1) * m_stepMax / m_valueMax;
+    int step = m_steps - (m_valueMax - m_value - 1) * m_steps / m_valueMax;
+
+    if (m_valueMax > 100 && qRound(percentage(stepToValue(step))) <= qRound(percentage(m_value))) {
+        // When no visible change was made, add 1 step.
+        // This can happen only if valueMax > 100, else 1 >= 1%.
+        step++;
+    }
+
     return stepToValue(step);
 }
 
-int BrightnessLogic::decreased() const {
+int BrightnessLogic::decreased() const
+{
     if (m_value == 0) {
         return -1; // ignore, we are at the minimum
     }
 
     // Subtract 1 and round downwards to the nearest Step
-    int step = (m_value - 1) * m_stepMax / m_valueMax;
+    int step = (m_value - 1) * m_steps / m_valueMax;
+
+    if (m_valueMax > 100 && qRound(percentage(stepToValue(step))) >= qRound(percentage(m_value))) {
+        // When no visible change was made, subtract 1 step.
+        // This can happen only if valueMax > 100, else 1 >= 1%.
+        step--;
+    }
+
     return stepToValue(step);
 }
 
-int BrightnessLogic::toggled() const {
+int BrightnessLogic::toggled() const
+{
     // If it's not minimum, set to minimum, if it's minimum, set to maximum
     return m_value > 0 ? 0 : m_valueMax;
 }
 
-int BrightnessLogic::value() const {
+int BrightnessLogic::value() const
+{
     return m_value;
 }
 
-int BrightnessLogic::valueMax() const {
+int BrightnessLogic::valueMax() const
+{
     return m_valueMax;
 }
 
-int BrightnessLogic::step() const {
-    return m_step;
+int BrightnessLogic::steps() const
+{
+    return m_steps;
 }
 
-int BrightnessLogic::stepMax() const {
-    return m_stepMax;
+float BrightnessLogic::percentage(int value) const
+{
+    return value * 100.0 / m_valueMax;
 }
 
-float BrightnessLogic::percentage() const {
-    return m_value * 100.0 / m_valueMax;
+const BrightnessLogic::BrightnessInfo BrightnessLogic::info() const
+{
+    return BrightnessInfo{
+        m_value, m_valueMax, m_steps
+    };
 }
 
-const BrightnessLogic::BrightnessInfo BrightnessLogic::info() const {
-    BrightnessInfo brightnessInfo;
-    brightnessInfo.valueMax = m_valueMax;
-    brightnessInfo.value = m_value;
-    brightnessInfo.stepMax = m_stepMax;
-    brightnessInfo.step = m_step;
-    brightnessInfo.percentage = percentage();
-    return brightnessInfo;
+int BrightnessLogic::stepToValue(int step) const
+{
+    return qBound(0, qRound(step * 1.0 * m_valueMax / m_steps), m_valueMax);
 }
 
-int BrightnessLogic::stepToValue(int step) const {
-    return qBound(0, qRound(step * 1.0 * m_valueMax / m_stepMax), m_valueMax);
+int BrightnessLogic::valueToStep(int value) const
+{
+    return qBound(0, qRound(value * 1.0 * m_steps / m_valueMax), m_steps);
 }
 
-int BrightnessLogic::valueToStep(int value) const {
-    return qBound(0, qRound(value * 1.0 * m_stepMax / m_valueMax), m_stepMax);
 }
-
-};
\ No newline at end of file
